@@ -754,6 +754,16 @@ InitExtensions(argc, argv)
 	/* Sort the extensions according the init dependencies. */
 	LoaderSortExtensions();
 	listInitialised = TRUE;
+    } else {
+	/* Call the setup functions on subsequent server resets as well */
+	for (i = 0; ExtensionModuleList[i].name != NULL; i++) {
+	    ext = &ExtensionModuleList[i];
+	    if (ext->setupFunc != NULL &&
+		(ext->disablePtr == NULL ||
+		 (ext->disablePtr != NULL && !*ext->disablePtr))) {
+		(ext->setupFunc)();
+	    }
+	}
     }
 
     for (i = 0; ExtensionModuleList[i].name != NULL; i++) {
